package es.ua.jtech.proyint.dao.usuario;

import java.util.List;

import es.ua.jtech.proyint.dao.DAOException;
import es.ua.jtech.proyint.to.UsuarioTO;

/**
 * Interface para el DAO Usuario
 * @author Miguel
 */
public interface IUsuarioDAO {
	
	/**
	 * Selecciona un usuario, comprobando su contrasea
	 * @param login Login del usuario a seleccionar
	 * @param password Contrasea dem
	 * @return El usuario seleccionado o null si no existe
	 * @throws DAOException
	 */
	UsuarioTO selectUsuario(String login, String password) throws DAOException;

	/**
	 * Selecciona un usuario, comprobando su contrasea
	 * @param login Login del usuario a seleccionar
	 * @return El usuario seleccionado o null si no existe
	 * @throws DAOException
	 */
	UsuarioTO selectUsuario(String login) throws DAOException;
	
	/**
	 * Aade un usuario a la BD
	 * @param usuario Usuario a aadir en la BD
	 * @throws DAOException
	 */
	void addUsuario(UsuarioTO usuario) throws DAOException;
	
	/**
	 * Borra el usuario de la BD
	 * @param login Usuario a eliminar
	 * @return Nmero de registros afectados en el borrado
	 * @throws DAOException
	 */
	int delUsuario(String login) throws DAOException;


	/**
	 * Modifica los datos de un usario de la BD
	 * @param usuario Usuario a modificar
	 * @return Nmero de registros afectados en la actualizacion
	 * @throws DAOException
	 */
	int updateUsuario(UsuarioTO usuario) throws DAOException;
	
	/**
	 * Devuelve una lista con todos los usuarios en la BD
	 * @return Lista con todos los usuarios
	 * @throws DAOException
	 */
	List<UsuarioTO> getAllUsuarios() throws DAOException;

	List<UsuarioTO> getPosiblesPrestatarios() throws DAOException;

}
